
const maxInjectionPerPacket = 8;

var cachedInjections = [];

var initInjectionEditor = function(){
    templateObj.$.ti_widget_droplist_injection_type.selectedIndex = -1;
    
    templateObj.$.ti_widget_numberbox_injection_offset.value = 0;
    templateObj.$.ti_widget_numberbox_injection_offset.disabled = true;
    
    templateObj.$.ti_widget_droplist_injection_counter_number.selectedIndex = 0;
    templateObj.$.ti_widget_droplist_injection_counter_number.disabled = true;
    
    templateObj.$.ti_widget_listbox_injection_list.labels = "";
};


var onInjectionSelect = function() {
    templateObj.$.ti_widget_numberbox_injection_offset.value = 0;
    templateObj.$.ti_widget_numberbox_injection_offset.disabled = true;
    
    templateObj.$.ti_widget_droplist_injection_counter_number.selectedIndex = 0;
    templateObj.$.ti_widget_droplist_injection_counter_number.disabled = true;

    switch ( templateObj.$.ti_widget_droplist_injection_type.selectedIndex ) {
        case 0: // Counter 1 through counter 8, binary format
        case 1: // Counter 1 through counter 8, ASCII format
            templateObj.$.ti_widget_numberbox_injection_offset.disabled = false;
            templateObj.$.ti_widget_droplist_injection_counter_number.disabled = false;
            break;
        case 2: // Device URN (ASCII)
        case 3: // Device MAC address (Raw)
        case 4: // Device MAC address (ASCII)
        case 5: // Date and time (internal, ASCII)
        case 6: // Heb Reg 2
        case 7: // Heb Reg 3
            templateObj.$.ti_widget_numberbox_injection_offset.disabled = false;
            break;
    }

};

var onAddInjection = function() {
    if (cachedInjections.length >= maxInjectionPerPacket) {
        alert("Max number of injection is : " + maxInjectionPerPacket.toString() + ", delete some and try again");
        return;
    }

    var selectedIndex = templateObj.$.ti_widget_droplist_injection_type.selectedIndex;
    var selectedText = templateObj.$.ti_widget_droplist_injection_type.selectedText;
    
    var injection = { };

    switch ( selectedIndex ) {
        case 0: // Counter 1 through counter 8, binary format
            injection.counter_number = templateObj.$.ti_widget_droplist_injection_counter_number.selectedIndex;
            injection.opcode = "1" + (injection.counter_number + 1).toString();
            break;
            
        case 1: // Counter 1 through counter 8, ASCII format
            injection.counter_number = templateObj.$.ti_widget_droplist_injection_counter_number.selectedIndex;
            injection.opcode = "2" + (injection.counter_number + 1).toString();
            break;
            
        case 2: // Device URN (ASCII)
            injection.opcode = "01";
            break;
            
        case 3: // Device MAC address (Raw)
            injection.opcode = "02";
            break;
            
        case 4: // Device MAC address (ASCII)
            injection.opcode = "03";
            break;        
            
        case 5: // Date and time (internal, ASCII)
            injection.opcode = "04";
            break;

        case 6: // Hib Reg 2
            injection.opcode = "05";
            break;
            
        case 7: // Hib Reg 3
            injection.opcode = "06";
            break;
    }
    
    injection.name = selectedText
    injection.offset = templateObj.$.ti_widget_numberbox_injection_offset.value;

    cachedInjections.push(injection);
    // console.log(cachedInjections);
    createInjectionList(cachedInjections);
};

var createInjectionList = function(arr) {
    templateObj.$.ti_widget_listbox_injection_list.labels = "";
    
    for (var n = 0; n < arr.length; n++) {
        var new_label = "";
        new_label += arr[n].name + " : " + arr[n].opcode;
        new_label += " \\ Offset [d] : " + arr[n].offset;
        
        if (arr[n].counter_number !== undefined) {
            new_label += " \\ Counter # : " + arr[n].counter_number;
        }

        templateObj.$.ti_widget_listbox_injection_list.labels += new_label + '|';
    }
};

var getInjectionList = function() {
    var ret = []
    var all_labels = templateObj.$.ti_widget_listbox_injection_list.labels;
    var all_labels_list = all_labels.split('|');
    
    all_labels_list.forEach(function(label){
        if (label) {
            var obj = {}
            
            var entry = label.split("\\");
            
            var name_split = entry[0].split(":");
            obj.name = name_split[0].trim();
            obj.opcode = name_split[1].trim();
            
            var offset_split = entry[1].split(":");
            obj.offset = parseInt(offset_split[1].trim());
            
            if (entry.length > 2) /*counter*/{
                var counter_number_split = entry[2].split(":");
                obj.counter_number = counter_number_split[1].trim();
            }
            
            ret.push(obj);
        }
    });
    
    return ret;
};

var onDeleteSelectedInjection = function() {
    var index = templateObj.$.ti_widget_listbox_injection_list.selectedIndex;
    
    if (index !== -1) {
        cachedInjections = getInjectionList();
        cachedInjections.splice(index, 1);
        createInjectionList(cachedInjections);
    }
};

var onDeleteAllSelectedInjection = function() {
    cachedInjections = [];
    templateObj.$.ti_widget_listbox_injection_list.labels = "";
};


var save_injection_data_into_array = function(arr, prefix){

    cachedInjections = getInjectionList();
    
    cachedInjections.forEach(function(injection, index){
        var injection_data = {
            "name" : prefix + "_injection_data_" + index.toString(),
            "raw_val" : injection,
            "val" : "0x" + zeroPad(injection.opcode,4) + zeroPad(injection.offset.toString(16),4),
            "size" : 4
        };
        arr.push(injection_data);
    });
};


var load_injection_data_from_array = function(arr, prefix) {
    var index = 0;

    cachedInjections = [];
    
    var injection_data = get_value_by_name(arr, prefix + "_injection_data_" + index.toString());

    while (injection_data !== "") {
        cachedInjections.push(injection_data);
        index++;
        injection_data = get_value_by_name(arr, prefix + "_injection_data_" + index.toString());
    }

    createInjectionList(cachedInjections);
};


